/*	$OpenBSD: atomic.h,v 1.3 2011/03/23 16:54:36 pirofti Exp $	*/

/* Public Domain */

#ifndef _MACHINE_ATOMIC_H_
#define _MACHINE_ATOMIC_H_

#include <powerpc/atomic.h>

#endif /* _MACHINE_ATOMIC_H_ */
